@@ -35,6 +35,7 @@
 #include "cond.h"
 #include "pid.h"
 #include "service.h"
+#include "sm.h"
 
 struct cond_boot {
 	TAILQ_ENTRY(cond_boot) link;
@@ -112,7 +113,7 @@ static void cond_boot_strap(void)
  *
  *    => 'pid/' + 'teamd' + '' = condition <pid/teamd>
  *
- * When you add a second aggrate you need to tell Finit it's a different
+ * When you add a second aggregate you need to tell Finit it's a different
  * instance usugin the :ID syntax:
  *
  *    service :lag2 teamd -f /etc/teamd/lag2.conf -- Aggregate lag2
@@ -214,7 +215,10 @@ static int do_delete(const char *fpath, const struct stat *sb, int tflag, struct
 		err(1, "Failed removing condition %s", fpath);
 
 	cond = ptr + strlen(COND_BASE) + 1;
-	cond_update(cond);
+	if (sm_is_in_teardown(&sm))
+		cond_clear_noupdate(cond);
+	else
+		cond_update(cond);
 
 	return 0;
 }
@@ -284,7 +288,7 @@ int cond_update(const char *name)
 	svc_t *svc, *iter = NULL;
 	int affects = 0;
 
-	dbg("%s", name);
+//	dbg("%s", name);
 	for (svc = svc_iterator(&iter, 1); svc; svc = svc_iterator(&iter, 0)) {
 		if (!svc_has_cond(svc) || !cond_affects(name, svc->cond))
 			continue;
@@ -317,7 +321,7 @@ int cond_set_noupdate(const char *name)
 
 void cond_set(const char *name)
 {
-	dbg("%s", name);
+//	dbg("%s", name);
 	if (cond_set_noupdate(name))
 		return;
 
@@ -368,7 +372,7 @@ int cond_clear_noupdate(const char *name)
 
 void cond_clear(const char *name)
 {
-	dbg("%s", name);
+//	dbg("%s", name);
 	if (cond_clear_noupdate(name))
 		return;
 
